; MOT_Syn3.asm - zen synchronnho motoru na mont M. Rottenborna (chod)
; AT89C2051 (Krystal 12 MHz)
; P3.0-P3.3 vstup pro synchonn motor - 50 Hz
; P3.4-5 - kontroln led: 
;   LED1 (erven) blik pi P_HVEZDY,
;   LED2 (zelen)  blik pi P_MESIC,
;   Pi rychlm pohybu zelen svt a erven blik
; P1.0-3 - vstupy pro nastaven rzn rychlosti

; FREK=65535-(1000000/F), kde F je poadovan frekvence peruen v Hz
ZRYCHLENI EQU 64535 ; kadch 1 kHz zrychl / zpomal o 1
;ZRYCHLENI EQU 55535 ; kadch 100 Hz zrychl / zpomal o 1

F_HVEZDY  EQU 63035 ; peruen 400 Hz, motrek  50 Hz
;F_MESIC   EQU 62954 ; peruen 388 Hz, motrek  48.44 Hz, chyba 0.02 procenta
F_MESIC   EQU 64910 ; Pokusn - peruen 1600 Hz, motrek 4x50 Hz
F_MAX     EQU 64285 ; peruen 800 Hz, motrek 2x50 Hz

; vstupy pro nastaven rzn rychlosti
P_HVEZDY  BIT P1.0  
P_MESIC   BIT P1.1
P_MIN     BIT P1.2
P_MAX     BIT P1.3

; vstupy LED
LED1      BIT P3.5  ; erven LED blikajc na f=INT/128 (0.64 s pi 50 Hz)
LED2      BIT P3.4  ; zelen  LED blikajc na f=INT/128 (0.64 s pi 50 Hz)

; Vyuit registr:
KROK      EQU R0    ; krok motrku (0-7)
DELICKA   EQU R1    ; dlika INT, aby LED mohla blikat pomaleji
L_FR_AKT  EQU R2    ; ni byte pedvyplnn asovae - aktuln rychlost
H_FR_AKT  EQU R3    ; vy byte pedvyplnn asovae - aktuln rychlost
L_FR_NAST EQU R4    ; ni byte pedvyplnn asovae - nastaven rychlost
H_FR_NAST EQU R5    ; vy byte pedvyplnn asovae - nastaven rychlost

FUNKCE    EQU B     ; funkce asovae (HVEZDY,MESIC,Rychl pohyb (FN_*))

; Konstanta ukldan do FUNKCE (reg. B)
FN_HVEZDY EQU 0     ; pohyb za hvzdama
FN_MESIC  EQU 1     ; pohyb za Mscem
FN_RYCHLY EQU 2     ; rychl pohyb
FN_STOJI  EQU 3     ; rychl pohyb

;---------------------------------------------------------------------
;---------------------------------------------------------------------
;---------------------------------------------------------------------

        ORG RESET
        JMP START

        ORG TIMER0                    ; obsluha preruseni od casovace0
        JMP PRERUS                    ; pulz do motrku

        ORG TIMER1                    ; obsluha preruseni od casovace1
        JMP RYCHL                     ; zmna rychlosti

;---------------------------------------------------------------------
        ORG 30H
START:  MOV TMOD,#00010001B           ; casovac 0 i 1 v modu 1
        MOV L_FR_NAST,#low  F_HVEZDY  ; pednastaven rychlost
        MOV H_FR_NAST,#high F_HVEZDY  ; pednastaven rychlost
        MOV L_FR_AKT ,#low  F_HVEZDY  ; uchovn rychlosti
        MOV H_FR_AKT ,#high F_HVEZDY  ; uchovn rychlosti
        MOV TL0,L_FR_AKT
        MOV TH0,H_FR_AKT
        MOV TL1,#low  ZRYCHLENI
        MOV TH1,#high ZRYCHLENI
        MOV IP,#2                     ; Priorita peruen od T0 je vy
        SETB EA                       ; povolen peruen
        SETB ET0                      ; povolen peruen od asovae 0
        SETB TR0                      ; sputn asovae
        SETB ET1                      ; povolen peruen od asovae 1
        SETB TR1                      ; sputn asovae
        MOV KROK,#0                   ; cislo kroku (0-3)
        MOV DELICKA,#0                ; dal dlika pro blikn LED
        MOV DPTR,#KROKY               ; adresa pamti uloen krok
;---------------------------------------------------------------------
ZNOVU:  
        JNB P_HVEZDY,TL01
        MOV L_FR_NAST,#low  F_HVEZDY
        MOV H_FR_NAST,#high F_HVEZDY
        MOV FUNKCE,#FN_HVEZDY
        JMP ZNOVU
TL01:
        JNB P_MESIC,TL02
        MOV L_FR_NAST,#low  F_MESIC
        MOV H_FR_NAST,#high F_MESIC
        MOV FUNKCE,#FN_MESIC
        JMP ZNOVU
TL02:
        JNB P_MIN,TL03
        MOV L_FR_NAST,#low  F_HVEZDY
        MOV H_FR_NAST,#high F_HVEZDY
        MOV L_FR_AKT ,#low  F_HVEZDY
        MOV H_FR_AKT ,#high F_HVEZDY
        MOV FUNKCE,#FN_STOJI
        JMP ZNOVU
                
TL03:
        JNB P_MAX,TL04
        MOV L_FR_NAST,#low  F_MAX
        MOV H_FR_NAST,#high F_MAX
        MOV FUNKCE,#FN_RYCHLY
        JMP ZNOVU
                
TL04:
        JMP ZNOVU
;---------------------------------------------------------------------
;---------------------------------------------------------------------
; Peruen od asovae 0 (mn registry ACC, PSW)
PRERUS: PUSH ACC
        PUSH PSW
        INC DELICKA

;-------------------------        
; Pulzy do motrku        
;-------------------------        
        MOV A,FUNKCE
        CLR C
        SUBB A,#FN_STOJI
        JZ PR0                ; motrek stoj, KROK A=0

        MOV A,KROK            ; KROK je citac kroku (0-7)
        INC A
        ANL A,#111B
        MOV KROK,A
        
PR0:    MOVC A,@A+DPTR
        MOV P3,A
;-------------------------        
; Blikn LED
;-------------------------        
        SETB LED1             ; zhasnut LED1
        SETB LED2             ; zhasnut LED2

        JNB FUNKCE.1,PR2
; rychl pohyb - zalen svt, erven blik
        CLR LED1              ; rozsvcen LED1
        CLR LED2              ; rozsvcen LED2

; normln pohyb - jedna blik
PR2:    MOV A,DELICKA

        JB FUNKCE.0,PR3

        RLC A
        RLC A
        MOV LED1,C            ; LED blikajc na f=INT/128 (0.64 s pi 50 Hz)
        JMP PR5

PR3:    RLC A
        RLC A
        MOV LED2,C            ; LED blikajc na f=INT/128 (0.64 s pi 50 Hz)
;-------------------------        
PR5:    MOV TL0,L_FR_AKT
        MOV TH0,H_FR_AKT
        POP PSW
        POP ACC
        RETI
;---------------------------------------------------------------------
;---------------------------------------------------------------------
; Peruen od asovae 1 - zmna rychlosti (mn registry ACC, PSW)
RYCHL:  MOV TL1,#low  ZRYCHLENI
        MOV TH1,#high ZRYCHLENI

        PUSH ACC
        PUSH PSW
        
        MOV A,H_FR_AKT         ; porovnn vych Byte
        CLR C
        SUBB A,H_FR_NAST
        JNZ R_High

        MOV A,L_FR_AKT         ; porovnn nich Byte, vy jsou stejn        
        CLR C
        SUBB A,L_FR_NAST
        JZ R_RET
        
R_High: JC RH1

        MOV A,L_FR_AKT
        DEC L_FR_AKT           ; Aktuln je vt
        JNZ R_RET
        DEC H_FR_AKT
        JMP R_RET

RH1:    INC L_FR_AKT           ; Aktuln je men
        MOV A,L_FR_AKT
        JNZ R_RET
        INC H_FR_AKT
        JMP R_RET

R_RET:  
        POP PSW
        POP ACC
        RETI
;---------------------------------------------------------------------
;---------------------------------------------------------------------
; Kroky pro synchronn motrek: 1 2 | 3 4 (PNP1,NPN1 | PNP2,NPN2)
KROKY:  DB 0000B              ;   - |   -  ve zaven
        DB 0011B              ;   - | +
        DB 0011B              ;   - | +
        DB 0011B              ;   - | +
        DB 0000B              ;   - |   -  ve zaven
        DB 1100B              ; +   |   -
        DB 1100B              ; +   |   -
        DB 1100B              ; +   |   -

;---------------------------------------------------------------------
; Popis a Copyright
        DB '   (C) 2002 Jiri Polak, Mot_syn3.asm - rizeni krokoveho moturku na montaz M. Rottenborna   '

END